package App::YTDL;

use warnings;
use strict;
use 5.010000;

our $VERSION = '0.031';


1;


__END__

=pod

=encoding UTF-8

=head1 NAME

App::YTDL - Download YouTube videos.

=head1 VERSION

Version 0.031

=cut

=head1 DESCRIPTION

See L<yt-download> for further information.

=head1 AUTHOR

Kuerbis <cuer2s@gmail.com>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2013-2015 Kuerbis.

This program is free software; you can redistribute it and/or modify it under the same terms as Perl 5.10.0.
For details, see the full text of the licenses in the file LICENSE.

=cut
